/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:RulesManagerModelLocator.as 2485 2008-09-30 14:14:35Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.enginedashboard.model.presentation
{
import flash.events.Event;
import flash.events.EventDispatcher;
import flash.events.TimerEvent;
import flash.utils.Timer;

import mx.binding.utils.*;
import mx.controls.Button;
import mx.resources.ResourceManager;
import mx.controls.Alert;

import org.ow2.jasmine.monitoring.eos.rules.enginedashboard.event.*;
import org.ow2.jasmine.monitoring.eos.rules.enginedashboard.model.domain.EngineDashboardModel;

/**
* Presentation model for the rule engine dashboard view
*
* @author Julien Nicoulaud
*/
[Bindable]
public class EngineDashboardPM extends EventDispatcher
{

    // =========================================================================
    // Variables
    // =========================================================================

  /**
  * Domain model
  */
  public var engineDashboardModel:EngineDashboardModel;

  /**
  * Timer used to refresh data
  */
  private var timer:Timer;

  /**
  * Marks the timer as stopped on purpose
  */
  public var disconnectedOnPurpose : Boolean = false;

  /**
  * The presentation state of the view layer
  */
  public var presentationState:String;

    /**
    * The status text displayed
    */
    public var statusText:String;

  /**
  * Component presentation presentationStates names
  * Must fit the server-side corresponding enum !
  */
  public const DISCONNECTED:String = "DISCONNECTED";
  public const CONNECTING:String = "CONNECTING";
  public const CONNECTED:String = "CONNECTED";
  public const IDLE:String = "IDLE";
  public const RUNNING:String = "RUNNING";
  public const STOPPED:String = "STOPPED";


    // =========================================================================
    // Constructors
    // =========================================================================

    /**
    * Build the Presentation Model for an engine dashboard view
    */
  public function EngineDashboardPM(engineDashboardModel:EngineDashboardModel)
  {
    // Set the model watched
    this.engineDashboardModel = engineDashboardModel;

    // Presentation state
    presentationState = CONNECTING;

        // Initialize the status text
        statusText = 'Connecting to the rule engine...';

      // Start the refresh timer
        startTimer(engineDashboardModel.refreshInterval);

        // Watch some model variables changes
        ChangeWatcher.watch(this.engineDashboardModel,"engineStatus",engineStatusChangeHandler);
        ChangeWatcher.watch(this.engineDashboardModel,"refreshInterval",refreshIntervalChangeHandler);

  }


    // =========================================================================
    // Domain model watching
    // =========================================================================

  /**
  * Called when the model engine status object changes
  */
  private function engineStatusChangeHandler(e:Event):void
  {
    // If disconnected, stop refreshing and switch state
    if(engineDashboardModel.engineStatus == null
    || engineDashboardModel.engineStatus.engineStatus == DISCONNECTED) {
      stopTimer();
      presentationState = DISCONNECTED;
    }

    // If connected, switch state
    else {
      // If previously disconnected, restart refreshing
      if(this.presentationState == DISCONNECTED) {
        startTimer(engineDashboardModel.refreshInterval);
      }

      presentationState = engineDashboardModel.engineStatus.engineStatus;
    }

    // Update the status text
    switch (presentationState) {
        case DISCONNECTED:
            statusText = ResourceManager.getInstance().getString('enginedashboardResources', 'enginestatusDisconnected.text');
            break;
        case CONNECTING:
            statusText = ResourceManager.getInstance().getString('enginedashboardResources', 'enginestatusConnecting.text');
            break;
        case CONNECTED:
            statusText = ResourceManager.getInstance().getString('enginedashboardResources', 'enginestatusConnected.text');
            break;
        case IDLE:
            statusText = ResourceManager.getInstance().getString('enginedashboardResources', 'enginestatusIdle.text');
            break;
        case RUNNING:
            statusText = ResourceManager.getInstance().getString('enginedashboardResources', 'enginestatusRunning.text');
            break;
        case STOPPED:
            statusText = ResourceManager.getInstance().getString('enginedashboardResources', 'enginestatusStopped.text');
            break;
        default:
            statusText = '';
        break;
    }

  }

    /**
    * Called when the model refresh interval changes
    */
    private function refreshIntervalChangeHandler(e:Event):void
    {
        startTimer(engineDashboardModel.refreshInterval);
    }


    // =========================================================================
    // Public methods
    // =========================================================================

  /**
  * Calls the "update engine status" command
  */
  public function updateEngineStatus():void
  {
      new EngineDashboardEvent(EngineDashboardEvent.UPDATE_ENGINE_STATUS + engineDashboardModel.componentID).dispatch();
  }

    /**
    * Calls the "start engine" command
    */
    public function startEngine():void
    {
        new EngineDashboardEvent(EngineDashboardEvent.START_ENGINE + engineDashboardModel.componentID).dispatch();
    }

    /**
    * Calls the "stop engine" command
    */
    public function stopEngine():void
    {
        new EngineDashboardEvent(EngineDashboardEvent.STOP_ENGINE + engineDashboardModel.componentID).dispatch();
    }

    /**
    * Deactivate a button
    */
    public function deactivateButton(button:Button):void {
      button.enabled = false;

    }

    /**
    * Disconnect from the server side
    */
    public function disconnect():void
    {
        stopTimer();
        disconnectedOnPurpose = true;
        presentationState = DISCONNECTED;
    }

    // =========================================================================
    // Private methods
    // =========================================================================

    /**
    * Start or restart the refresh with a given refresh interval
    */
    private function startTimer(refreshInterval:int):void
    {
        // First stop the timer if needed
        stopTimer();

        // Recreate it and start it
        timer = new Timer(refreshInterval);
        timer.addEventListener("timer", timerHandler);
        disconnectedOnPurpose = false;
        timer.start();
    }

    /**
    * Stop the refresh timer
    */
    private function stopTimer():void
    {
        // If the timer was started, destroy it
        if(timer != null) {
          if (timer.running) {
                timer.stop();
          }
            timer = null;
        }
    }

    /**
    * Called every time the timer completes an interval of time
    */
  private function timerHandler(event:TimerEvent):void
  {
      updateEngineStatus();
    }

}
}
